@@ -15,7 +15,7 @@ from Pyro4.errors import NamingError, PyroError, ProtocolError
 from Pyro4 import core, socketutil
 import Pyro4.constants
 
-__all__ = ["locateNS", "resolve", "startNS"]
+__all__ = ["locateNS", "resolve", "startNS", "startNSloop", "MemoryStorage"]
 
 if sys.version_info >= (3, 0):
     basestring = str
@@ -23,17 +23,53 @@ if sys.version_info >= (3, 0):
 log = logging.getLogger("Pyro4.naming")
 
 
-class NameServer(object):
-    """Pyro name server. Provides a simple flat name space to map logical object names to Pyro URIs."""
+class MemoryStorage(dict):
+    """
+    Storage implementation that is just an in-memory dict.
+    Stopping the nameserver will make the server instantly forget about everything.
+    """
+    def __init__(self, **kwargs):
+        super(MemoryStorage, self).__init__(**kwargs)
 
-    def __init__(self):
-        self.namespace = {}
+    def optimized_prefix_list(self, prefix):
+        return None
+
+    def optimized_regex_list(self, regex):
+        return None
+
+    def everything(self):
+        return self.copy()
+
+    def remove_items(self, items):
+        for item in items:
+            try:
+                del self[item]
+            except KeyError:
+                pass
+
+    def close(self):
+        pass
+
+
+class NameServer(object):
+    """
+    Pyro name server. Provides a simple flat name space to map logical object names to Pyro URIs.
+    Default storage is done in an in-memory dictionary. You can provide custom storage types.
+    """
+    def __init__(self, storageProvider=None):
+        self.storage = storageProvider
+        if storageProvider is None:
+            self.storage = MemoryStorage()
+            log.debug("using volatile in-memory dict storage")
         self.lock = RLock()
 
+    def count(self):
+        return len(self.storage)
+
     def lookup(self, name):
         """Lookup the given name, returns an URI if found"""
         try:
-            return core.URI(self.namespace[name])
+            return core.URI(self.storage[name])
         except KeyError:
             raise NamingError("unknown name: " + name)
 
@@ -48,47 +84,50 @@ class NameServer(object):
             core.URI(uri)  # check if uri is valid
         if not isinstance(name, basestring):
             raise TypeError("name must be a str")
-        if safe and name in self.namespace:
-            raise NamingError("name already registered: " + name)
         with self.lock:
-            self.namespace[name] = uri
+            if safe and name in self.storage:
+                raise NamingError("name already registered: " + name)
+            self.storage[name] = uri
 
     def remove(self, name=None, prefix=None, regex=None):
         """Remove a registration. returns the number of items removed."""
-        if name and name in self.namespace and name != Pyro4.constants.NAMESERVER_NAME:
+        if name and name in self.storage and name != Pyro4.constants.NAMESERVER_NAME:
             with self.lock:
-                del self.namespace[name]
+                del self.storage[name]
             return 1
         if prefix:
-            with self.lock:
-                items = list(self.list(prefix=prefix).keys())
-                if Pyro4.constants.NAMESERVER_NAME in items:
-                    items.remove(Pyro4.constants.NAMESERVER_NAME)
-                for item in items:
-                    del self.namespace[item]
-                return len(items)
+            items = list(self.list(prefix=prefix).keys())
+            if Pyro4.constants.NAMESERVER_NAME in items:
+                items.remove(Pyro4.constants.NAMESERVER_NAME)
+            self.storage.remove_items(items)
+            return len(items)
         if regex:
-            with self.lock:
-                items = list(self.list(regex=regex).keys())
-                if Pyro4.constants.NAMESERVER_NAME in items:
-                    items.remove(Pyro4.constants.NAMESERVER_NAME)
-                for item in items:
-                    del self.namespace[item]
-                return len(items)
+            items = list(self.list(regex=regex).keys())
+            if Pyro4.constants.NAMESERVER_NAME in items:
+                items.remove(Pyro4.constants.NAMESERVER_NAME)
+            self.storage.remove_items(items)
+            return len(items)
         return 0
 
+    # noinspection PyNoneFunctionAssignment
     def list(self, prefix=None, regex=None):
         """Retrieve the registered items as a dictionary name-to-URI. The URIs
         in the resulting dict are strings, not URI objects.
         You can filter by prefix or by regex."""
         with self.lock:
             if prefix:
+                result = self.storage.optimized_prefix_list(prefix)
+                if result is not None:
+                    return result
                 result = {}
-                for name in self.namespace:
+                for name in self.storage:
                     if name.startswith(prefix):
-                        result[name] = self.namespace[name]
+                        result[name] = self.storage[name]
                 return result
             elif regex:
+                result = self.storage.optimized_regex_list(regex)
+                if result is not None:
+                    return result
                 result = {}
                 try:
                     regex = re.compile(regex + "$")  # add end of string marker
@@ -96,13 +135,13 @@ class NameServer(object):
                     x = sys.exc_info()[1]
                     raise NamingError("invalid regex: " + str(x))
                 else:
-                    for name in self.namespace:
+                    for name in self.storage:
                         if regex.match(name):
-                            result[name] = self.namespace[name]
+                            result[name] = self.storage[name]
                     return result
             else:
                 # just return (a copy of) everything
-                return self.namespace.copy()
+                return self.storage.everything()
 
     def ping(self):
         """A simple test method to check if the name server is running correctly."""
@@ -112,7 +151,7 @@ class NameServer(object):
 class NameServerDaemon(core.Daemon):
     """Daemon that contains the Name Server."""
 
-    def __init__(self, host=None, port=None, unixsocket=None, nathost=None, natport=None):
+    def __init__(self, host=None, port=None, unixsocket=None, nathost=None, natport=None, storage=None):
         if host is None:
             host = Pyro4.config.HOST
         if port is None:
@@ -121,15 +160,35 @@ class NameServerDaemon(core.Daemon):
             nathost = Pyro4.config.NATHOST
         if natport is None:
             natport = Pyro4.config.NATPORT or None
+        storage = storage or "memory"
+        if storage == "memory":
+            log.debug("using volatile in-memory dict storage")
+            self.nameserver = NameServer(MemoryStorage())
+        elif storage.startswith("dbm:") and len(storage) > 4:
+            dbmfile = storage[4:]
+            log.debug("using persistent dbm storage in file %s", dbmfile)
+            from Pyro4.naming_storage import DbmStorage
+            self.nameserver = NameServer(DbmStorage(dbmfile))
+        elif storage.startswith("sql:") and len(storage) > 4:
+            sqlfile = storage[4:]
+            log.debug("using persistent sql storage in file %s", sqlfile)
+            from Pyro4.naming_storage import SqlStorage
+            self.nameserver = NameServer(SqlStorage(sqlfile))
+        else:
+            raise ValueError("invalid storage type '%s'" % storage)
+        existing_count = self.nameserver.count()
+        if existing_count > 0:
+            log.debug("number of existing entries in storage: %d", existing_count)
         super(NameServerDaemon, self).__init__(host, port, unixsocket, nathost=nathost, natport=natport)
-        self.nameserver = NameServer()
         self.register(self.nameserver, Pyro4.constants.NAMESERVER_NAME)
         self.nameserver.register(Pyro4.constants.NAMESERVER_NAME, self.uriFor(self.nameserver))
         log.info("nameserver daemon created")
 
     def close(self):
         super(NameServerDaemon, self).close()
-        self.nameserver = None
+        if self.nameserver is not None:
+            self.nameserver.storage.close()
+            self.nameserver = None
 
     def __enter__(self):
         if not self.nameserver:
@@ -137,6 +196,8 @@ class NameServerDaemon(core.Daemon):
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
+        if self.nameserver is not None:
+            self.nameserver.storage.close()
         self.nameserver = None
         return super(NameServerDaemon, self).__exit__(exc_type, exc_value, traceback)
 
@@ -228,9 +289,10 @@ class BroadcastServer(object):
         self.close()
 
 
-def startNSloop(host=None, port=None, enableBroadcast=True, bchost=None, bcport=None, unixsocket=None, nathost=None, natport=None):
+def startNSloop(host=None, port=None, enableBroadcast=True, bchost=None, bcport=None, unixsocket=None, nathost=None, natport=None, storage=None, hmac=None):
     """utility function that starts a new Name server and enters its requestloop."""
-    daemon = NameServerDaemon(host, port, unixsocket, nathost=nathost, natport=natport)
+    daemon = NameServerDaemon(host, port, unixsocket, nathost=nathost, natport=natport, storage=storage)
+    daemon._pyroHmacKey = hmac
     nsUri = daemon.uriFor(daemon.nameserver)
     internalUri = daemon.uriFor(daemon.nameserver, nat=False)
     bcserver = None
@@ -249,7 +311,12 @@ def startNSloop(host=None, port=None, enableBroadcast=True, bchost=None, bcport=
             bcserver = BroadcastServer(internalUri, bchost, bcport)
             print("Broadcast server running on %s" % bcserver.locationStr)
             bcserver.runInThread()
+    existing = daemon.nameserver.count()
+    if existing > 1:   # don't count our own nameserver registration
+        print("Persistent store contains %d existing registrations." % existing)
     print("NS running on %s (%s)" % (daemon.locationStr, hostip))
+    if not hmac:
+        print("Warning: HMAC key not set. Anyone can connect to this server!")
     if daemon.natLocationStr:
         print("internal URI = %s" % internalUri)
         print("external URI = %s" % nsUri)
@@ -264,10 +331,11 @@ def startNSloop(host=None, port=None, enableBroadcast=True, bchost=None, bcport=
     print("NS shut down.")
 
 
-def startNS(host=None, port=None, enableBroadcast=True, bchost=None, bcport=None, unixsocket=None, nathost=None, natport=None):
+def startNS(host=None, port=None, enableBroadcast=True, bchost=None, bcport=None, unixsocket=None, nathost=None, natport=None, storage=None, hmac=None):
     """utility fuction to quickly get a Name server daemon to be used in your own event loops.
     Returns (nameserverUri, nameserverDaemon, broadcastServer)."""
-    daemon = NameServerDaemon(host, port, unixsocket, nathost=nathost, natport=natport)
+    daemon = NameServerDaemon(host, port, unixsocket, nathost=nathost, natport=natport, storage=storage)
+    daemon._pyroHmacKey = hmac
     bcserver = None
     nsUri = daemon.uriFor(daemon.nameserver)
     if not unixsocket:
@@ -281,7 +349,7 @@ def startNS(host=None, port=None, enableBroadcast=True, bchost=None, bcport=None
     return nsUri, daemon, bcserver
 
 
-def locateNS(host=None, port=None, broadcast=True):
+def locateNS(host=None, port=None, broadcast=True, hmac_key=None):
     """Get a proxy for a name server somewhere in the network."""
     if host is None:
         # first try localhost if we have a good chance of finding it there
@@ -292,6 +360,7 @@ def locateNS(host=None, port=None, broadcast=True):
             uristring = "PYRO:%s@%s:%d" % (Pyro4.constants.NAMESERVER_NAME, host, port or Pyro4.config.NS_PORT)
             log.debug("locating the NS: %s", uristring)
             proxy = core.Proxy(uristring)
+            proxy._pyroHmacKey = hmac_key
             try:
                 proxy.ping()
                 log.debug("located NS")
@@ -346,6 +415,7 @@ def locateNS(host=None, port=None, broadcast=True):
     uri = core.URI(uristring)
     log.debug("locating the NS: %s", uri)
     proxy = core.Proxy(uri)
+    proxy._pyroHmacKey = hmac_key
     try:
         proxy.ping()
         log.debug("located NS")
@@ -356,7 +426,7 @@ def locateNS(host=None, port=None, broadcast=True):
         raise e
 
 
-def resolve(uri):
+def resolve(uri, hmac_key=None):
     """Resolve a 'magic' uri (PYRONAME) into the direct PYRO uri."""
     if isinstance(uri, basestring):
         uri = core.URI(uri)
@@ -366,7 +436,7 @@ def resolve(uri):
         return uri
     log.debug("resolving %s", uri)
     if uri.protocol == "PYRONAME":
-        nameserver = locateNS(uri.host, uri.port)
+        nameserver = locateNS(uri.host, uri.port, hmac_key=hmac_key)
         uri = nameserver.lookup(uri.object)
         nameserver._pyroRelease()
         return uri
@@ -380,6 +450,7 @@ def main(args=None):
     parser.add_option("-n", "--host", dest="host", help="hostname to bind server on")
     parser.add_option("-p", "--port", dest="port", type="int", help="port to bind server on (0=random)")
     parser.add_option("-u", "--unixsocket", help="Unix domain socket name to bind server on")
+    parser.add_option("-s", "--storage", help="Storage system to use (memory, dbm:file, sql:file)", default="memory")
     parser.add_option("", "--bchost", dest="bchost", help="hostname to bind broadcast server on (default is \"\")")
     parser.add_option("", "--bcport", dest="bcport", type="int",
                       help="port to bind broadcast server on (0=random)")
@@ -387,10 +458,12 @@ def main(args=None):
     parser.add_option("", "--natport", dest="natport", type="int", help="external port in case of NAT")
     parser.add_option("-x", "--nobc", dest="enablebc", action="store_false", default=True,
                       help="don't start a broadcast server")
+    parser.add_option("-k", "--key", help="the HMAC key to use")
     options, args = parser.parse_args(args)
     startNSloop(options.host, options.port, enableBroadcast=options.enablebc,
                 bchost=options.bchost, bcport=options.bcport, unixsocket=options.unixsocket,
-                nathost=options.nathost, natport=options.natport)
+                nathost=options.nathost, natport=options.natport, storage=options.storage,
+                hmac=options.key)
 
 
 if __name__ == "__main__":
